package top.xinsin.service;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import top.xinsin.mapper.AccountMapper;
import top.xinsin.mapper.AuthPermissionsMapper;
import top.xinsin.mapper.AuthRolesMapper;
import top.xinsin.mapper.AuthRolesPermissionsMapper;
import top.xinsin.pojo.AuthAccount;
import top.xinsin.pojo.AuthPermissions;
import top.xinsin.pojo.AuthRoles;
import top.xinsin.pojo.AuthRolesPermissions;
import top.xinsin.util.HttpCodes;
import top.xinsin.util.R;

import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * @author xinsin
 * Created On 2023/4/24 21:11
 * @version 1.0
 */
@Service
@Slf4j
public class AccountService implements UserDetailsService {
    @Autowired
    private AccountMapper accountMapper;
    @Autowired
    private AuthRolesPermissionsMapper authRolesPermissionsMapper;
    @Autowired
    private AuthRolesMapper authRolesMapper;
    @Autowired
    private AuthPermissionsMapper authPermissionsMapper;

    BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();

    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        if (username == null) {
            throw new UsernameNotFoundException("用户名不能为空");
        } else {
            AuthAccount authAccount = this.accountMapper.selectOne(new LambdaQueryWrapper<AuthAccount>()
                    .eq(AuthAccount::getUsername, username)
                    .or()
                    .eq(AuthAccount::getEmail, username)
                    .eq(AuthAccount::getStatus, true));
            if (authAccount == null) {
                throw new UsernameNotFoundException("用户名或密码错误");
            } else {
                return User
                        .withUsername(authAccount.getUsername())
                        .password(authAccount.getPassword())
                        .roles("id=" + authAccount.getId(),"roleId=" + authAccount.getRoleId())
                        .build();
            }
        }
    }

    public R<String> register(AuthAccount authAccount) {
        authAccount.setCreateTime(new Date());
        authAccount.setStatus(true);
        authAccount.setPassword(this.encoder.encode(authAccount.getPassword()));
        int insert = this.accountMapper.insert(authAccount);
        return insert == 1 ? R.success("注册成功") : R.failed(HttpCodes.HTTP_CODES555, "注册失败,您的用户名或者邮箱已经被注册过了");
    }

    public R<String> changeAccount(AuthAccount authAccount) {
        if (authAccount.getPassword() != null) {
            authAccount.setPassword(this.encoder.encode(authAccount.getPassword()));
        }

        int update = this.accountMapper.updateById(authAccount);
        return update == 1 ? R.success("修改用户信息成功") : R.failed(HttpCodes.HTTP_CODES555, "修改用户信息失败,请联系服务器管理员");
    }

    public R<String> changeStatus(Integer userId, Boolean status) {
        AuthAccount authAccount = new AuthAccount();
        authAccount.setId(userId);
        authAccount.setStatus(status);
        int i = this.accountMapper.updateById(authAccount);
        if (i == 1) {
            return status ? R.success("启用该用户成功") : R.success("禁用该用户成功");
        } else {
            return R.failed(HttpCodes.HTTP_CODES555, "修改用户状态失败");
        }
    }

    public R<String> changePassword(Integer userId, String oldPassword, String newPassword) {
        AuthAccount authAccount = new AuthAccount();
        authAccount.setPassword(this.encoder.encode(newPassword));
        LambdaQueryWrapper<AuthAccount> accountLambdaQueryWrapper = new LambdaQueryWrapper<>();
        accountLambdaQueryWrapper
                .eq(AuthAccount::getId, userId)
                .eq(AuthAccount::getPassword, this.encoder.encode(oldPassword));
        int update = this.accountMapper.update(authAccount, accountLambdaQueryWrapper);
        return update == 1 ? R.success("修改密码成功") : R.failed(HttpCodes.HTTP_CODES555, "修改密码失败,请联系服务器管理员");
    }

    public R<JSONObject> getUser(Integer page, Integer num) {
        Page<AuthAccount> accountPage = new Page<>(page, num);
        Page<AuthAccount> accountPage1 = this.accountMapper.selectPage(accountPage, null);
        JSONObject jsonObject = new JSONObject();
        List<AuthAccount> collect = accountPage1.getRecords().stream().peek((e) -> e.setPassword(null)).toList();
        jsonObject
                .fluentPut("info", collect)
                .fluentPut("total", accountPage1.getTotal());
        return R.success(jsonObject);
    }

    public R<String> delUser(Integer id) {
        int delete = this.accountMapper.deleteById(id);
        return delete == 1 ? R.success("删除用户成功") : R.failed(HttpCodes.HTTP_CODES555, "删除用户失败,请联系服务器管理员");
    }

    public R<JSONObject> getUserInfo(int rolesId) {
        System.out.println(rolesId);
        JSONObject globalReturnData = new JSONObject();
        JSONArray globalReturnPermissionList = new JSONArray();
        List<AuthRolesPermissions> authRolesPermissions = authRolesPermissionsMapper
                .selectList(new LambdaQueryWrapper<AuthRolesPermissions>()
                        .eq(AuthRolesPermissions::getRoleId, rolesId));
        List<Integer> permissionsList = authRolesPermissions.stream().map(AuthRolesPermissions::getPermissionId).toList();
        List<AuthPermissions> authPermissions = authPermissionsMapper.selectList(new LambdaQueryWrapper<AuthPermissions>()
                .in(AuthPermissions::getId, permissionsList));
        authPermissions.forEach(e -> {
            HashMap<String, String> returnPermissionJson = new HashMap<>();
            returnPermissionJson.put("permissionName",e.getPermissionName());
            returnPermissionJson.put("permissionCode",e.getPermissionCode());
            globalReturnPermissionList.add(returnPermissionJson);
        });
        AuthRoles authRoles = authRolesMapper.selectOne(new LambdaQueryWrapper<AuthRoles>().eq(AuthRoles::getId, rolesId));
//                角色名
        String roleName = authRoles.getRoleName();
        globalReturnData
                .fluentPut("info",globalReturnPermissionList)
                .fluentPut("roleName",roleName);
        return R.success(globalReturnData);
    }
}
